/*
 Portable Automatic Anti Ghoul

 (C) 2000-2002 by Piotr Winiarczyk, Version 2.0d

  This program can be used with VGA-Planets game.

  version 1.0
    strncpy          - done
    config file name - done
    versions in rst  - done

  version 1.1
    gLogFile,Info,OpenInputFile,..
    kill.log file is created if needed
    more information is displayed

  version 1.2
    23.03.2001
    minor changes
    documentation update
    24.03.2001
    compiles and runs on Linux, no warnings (-Wall)
    minor config reader changes

  version 1.3
     8.06.2001
    DOSWrite16 and DOSRead16 function used
    minor config reader changes
    new option : -s displays information from trns status file and exit

  version 2.0
        9.06.2001
        Two phase operation. Done, but not needed in standard usage.

    'a' 12.02.2002
        Added letter to version. Compile time to -v option.

    'b'
        Signature in used binary file. Better error handling in
        load and save operation.

    'c'
        Corrections in information displayed by program.

    'd' 2002-10-16 21:34
        Small corection of return code.
        0 - EXIT_SUCCES
        Code cleaning
        Turn 1 welcome message.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>

#include "phostpdk.h"

#define  AND   &&
#define  OR    ||
#define  EQ    ==
#define  NEQ   !=
#define  GT    >
#define  GE    >=
#define  LT    <
#define  LE    =<

#define  CONFIG_BOOL  0
#define  CONFIG_INT   1

#define  PARAM_NUMBER 10

#define  FILE_SIGNATURE 0x050414719

#define  MYNAME       "PAAGhoul"

char *strupr(char *);

// Config defaults
Boolean KillShips        = True;
Boolean KillPlanets      = True;
Boolean KillStructures   = True;
Boolean KillCash         = True;
Boolean KillMineFields   = True;
Boolean WarningMessages  = True;
Boolean KillMessages     = True;
Boolean KillPlayers      = True;
Boolean TrnInfoMessages  = True;

Uns16 TurnsNoSubmitted   = 5;

char configname[80]      = "paaghoul.src";
char logname[80]         = "paaghoul.log";
char ghoulfilename[80]   = "paaghoul.hst";
char killlog[80]         = "kill.log";

char MessHeadString[80];

// version number
Uns16 pMajor = 2;
Uns16 pMinor = 0;
char  pLetter='d';

// other options
Boolean printO   = False;
Boolean logF     = True;
Boolean disp     = False;
Boolean phase1   = True;
Boolean phase2   = True;

// loaded/saved to file
Uns16 phase1turn = 0;
Uns16 phase2turn = 0;
Uns16 TurnsNotIn[RACE_NR];
Uns16 KillInPhase2[RACE_NR];

Boolean ConfigError;

struct config_rec  {
              char    name[50];
              char    name_d[50];
              Uns16   mode;
              Boolean read;
                };

struct config_rec config[PARAM_NUMBER] =
            {
              {"KillShips"        ,"Yes" ,CONFIG_BOOL,False},
              {"KillPlanets"      ,"Yes" ,CONFIG_BOOL,False},
              {"KillStructures"   ,"Yes" ,CONFIG_BOOL,False},
              {"KillCash"         ,"Yes" ,CONFIG_BOOL,False},
              {"KillMineFields"   ,"Yes" ,CONFIG_BOOL,False},
              {"WarningMessages"  ,"Yes" ,CONFIG_BOOL,False},
              {"KillMessages"     ,"Yes" ,CONFIG_BOOL,False},
              {"TurnsNoSubmitted" ,"5"   ,CONFIG_INT ,False},
              {"KillPlayers"      ,"Yes" ,CONFIG_BOOL,False},
              {"TrnInfoMessages"  ,"Yes" ,CONFIG_BOOL,False}
            };

static void
version(void)
{
   Info("Portable Automatic Anti Ghoul v%u.%u%c\n",pMajor,pMinor,pLetter);

// head string for all messages
   sprintf(MessHeadString,"(-a000) "MYNAME" v%u.%u%c\r",pMajor,pMinor,pLetter);
}

static void
usage(void)
{
Info(
"Usage: "MYNAME" [option] [GameDirectory [RootDirectory]]\n"
"\n"
"options:\n"
"        -1      -- Run just phase 1. Look for trn and designate races to kill.\n"
"        -2      -- Run just phase 2. Kill races designated in phase 1.\n"
"        -c      -- Print command line strings.\n"
"        -f name -- Use file name as config file.\n"
"        -h      -- Print help and exit.\n"
"        -l      -- Don't create log file.\n"
"        -s      -- Display current trns status and exit.\n"
"        -v      -- Print version and exit.\n"
);
   exit(EXIT_SUCCESS);
}

static void processOptions(int argc, char *argv[])
{
   int c;

    while ((c=pgetopt(argc, argv, "12shHvclf:")) NEQ EOF) {
      switch (c) {
       case 'h': case 'H': default:
         usage();
       case 'v':
         version();
         Info("GCC version: %s",__VERSION__);
         Info("Compilation date & time: %s %s",__DATE__,__TIME__);
         exit(EXIT_SUCCESS);
       case 'c':
         printO=True;
         break;
       case 's':
         disp = True;
         break;
       case 'l':
         logF=False;
         break;
       case '1':
         phase1=True;
         phase2=False;
         break;
       case '2':
         phase1=False;
         phase2=True;
         break;
       case 'f':
         strncpy(configname,poptarg,80);
         break;
      }
   }

    if (poptind < argc) {
        gGameDirectory = argv[poptind++];
   }

    if (poptind < argc) {
        gRootDirectory = argv[poptind++];
   }

    if (poptind < argc) usage();

}

/*
    Kill race module based on killrace v 3.1 by Andrew Sterian
*/

int
killrace(Uns16 gKillRace)
{
    Uns16 lIx;
    Boolean lShipRemoved[SHIP_NR+1];
    char  FC[4];
    FILE *tmp;
    time_t ti;

    if (! PlayerIsActive(gKillRace)) return 1;
// zero array
    memset(lShipRemoved, 0, sizeof(lShipRemoved));

// logging to killllog file
    tmp=gLogFile;
    gLogFile=OpenOutputFile(killlog,GAME_DIR_ONLY|TEXT_MODE|APPEND_MODE);

    Info("Killing player    : %s",RaceNameAdjective(gKillRace,NULL));
    Info("Turn number       : %d",TurnNumber());
    ti=time(0);
    Info("Paaghoul run time : %s",ctime(&ti));

    if (KillShips)
        Info("Killing ships.");
    if (KillPlanets)
    {
        Info("Killing planets.");
    if (KillCash)
        Info("Killing cash.");
    if (KillStructures)
        Info("Killing structures.");
    }
    if (KillMineFields)
        Info("Killing minefields.");
    Info("Killing alliances.\n\n");
    fclose( gLogFile);
    gLogFile=tmp;
    Info("Player %2d killed.",gKillRace);

    // kill now
    for (lIx = 1; lIx <= SHIP_NR; lIx++) {
// ships
    if (KillShips)
        if (IsShipExist(lIx) AND ShipOwner(lIx) EQ gKillRace)
        {
            DeleteShip(lIx);
            ResetShipRemoteControl(lIx);
            lShipRemoved[lIx] = True;
        }
// planets
    if (KillPlanets)
      {
        if (IsPlanetExist(lIx) AND PlanetOwner(lIx) EQ gKillRace)
        {
            if (IsBaseExist(lIx))
            {
                BuildQueueInvalidate(lIx);
                DeleteBase(lIx);
            }
            PutPlanetCargo(lIx, COLONISTS, 0);
            PutPlanetOwner(lIx, NoOwner);
            PutPlanetColHappy(lIx,100);
// setting FC to random
            sprintf(FC, "%03u", RandomRange(1000));
            PutPlanetFC(lIx, FC);

       if (KillCash)
          {
            PutPlanetCargo(lIx, SUPPLIES, 0);
            PutPlanetCargo(lIx, CREDITS, 0);
           }
       if (KillStructures)
          {
            PutPlanetMines    (lIx,0);
            PutPlanetFactories(lIx,0);
            PutPlanetDefense  (lIx,0);
          }
        }  // planet exist
      } // kill planets
// mine fields
        if (KillMineFields)
        if (IsMinefieldExist(lIx) AND MinefieldOwner(lIx) EQ gKillRace )
           PutMinefieldUnits(lIx,0);
    } // kill now

// kill alliances
    for (lIx = 1; lIx <= RACE_NR; lIx++) {
        AllyDropRequest(gKillRace, lIx);
    }
    PutPlayerIsActive(gKillRace, False);

    /* Now go through and clear tow/intercept missions on ships which
       were towing/intercepting a ship that we just deleted. */
    if (KillShips)
    for (lIx = 1; lIx <= SHIP_NR; lIx++) {
        if (! IsShipExist(lIx)) continue;

        if (    (ShipMission(lIx) EQ Tow)
            AND lShipRemoved[ShipTowTarget(lIx)]
           ) {
            PutShipMission(lIx, NoMission);
        }
        if (    (ShipMission(lIx) EQ Intercept)
            AND lShipRemoved[ShipInterceptTarget(lIx)]
           ) {
            PutShipMission(lIx, NoMission);
        }
    }
    return 0;
}

void config_bool (int i, Boolean value)
{
  switch (i)
  {
   case 0: KillShips=value; break;
   case 1: KillPlanets=value; break;
   case 2: KillStructures=value; break;
   case 3: KillCash=value; break;
   case 4: KillMineFields=value; break;
   case 5: WarningMessages=value; break;
   case 6: KillMessages=value; break;
   case 8: KillPlayers=value; break;
   case 9: TrnInfoMessages=value; break;
  }
}

void config_int (int i, int value)
{
  switch (i)
  {
   case 7: TurnsNoSubmitted=value; break;
  }
}

Boolean my_func(const char *lhs, char *rhs, const char *lInputLine)
{
  int i,tmp_int,tmp_;
  char tmp1[80],tmp2[80];
  Boolean line_ok;

  if ((lhs==NULL)&&(rhs==NULL)) return True;

  line_ok=False;
  
  for (i=0;i<PARAM_NUMBER;i++)
  {
     strncpy(tmp1,lhs,80);
     strupr(tmp1);
     strcpy(tmp2,config[i].name);
     strupr(tmp2);
     if (strcmp(tmp1,tmp2)==0)
     {
        line_ok=True; 

        if (config[i].mode==CONFIG_BOOL)
        {
          strncpy(tmp1,rhs,80);
          strupr(tmp1);
          if ((strcmp(tmp1,"TRUE")==0)||(strcmp(tmp1,"YES")==0))
          {
            config_bool(i,True);
            config[i].read=True;
            break;
          }
          else
          if ((strcmp(tmp1,"FALSE")==0)||(strcmp(tmp1,"NO")==0))
          {
            config_bool(i,False);
            config[i].read=True;
            break;
          }
          else
// wrong value
           { line_ok=False; break; }
        }
        else
        if (config[i].mode==CONFIG_INT)
        {
           tmp_=sscanf(rhs,"%d",&tmp_int);
// check if wrong value
           if (tmp_==0) { line_ok=False; break; }

           config_int(i,tmp_int);
           config[i].read=True;
           break;
        }

     }  // strcmp
  } // params
  
 ConfigError=!line_ok;
  
 if (line_ok) return True;
 else         return False;
}


void ReadConfig(void)
{
   FILE *f;
   int i;
   Boolean tr;

   f= OpenInputFile(configname, GAME_DIR_ONLY|TEXT_MODE);
   if (f!=NULL) ConfigFileReader(f,configname,MYNAME,False,*my_func);
   fclose(f);
   
   if (ConfigError) ErrorExit("Config file cannot be processed.");
   
   tr=False;
   for (i=0;i<PARAM_NUMBER;i++)
   if (config[i].read==False) tr=True;
   if (tr==True)
   {
     Info("WARNING ! Those parameters are not in config file.\nSetting them to default.");
     for (i=0;i<PARAM_NUMBER;i++)
     if (config[i].read==False) Info("%s = %s",config[i].name,config[i].name_d);
     Info("");
   }
}

void printOptions(int argc, char *argv[])
{
  int i;

  Info("Command line strings :");
  for (i=0;i<argc;i++) Info("%s",argv[i]);
  Info("");
}

void LoadGhoulFile(void)
{
    FILE *ghoulfile;
    Uns32 sygnature;

    sygnature=FILE_SIGNATURE;
    ghoulfile=OpenInputFile(ghoulfilename,GAME_DIR_ONLY);
    if (ghoulfile!=NULL) {
        DOSRead32(&sygnature,1,ghoulfile);

        if (sygnature!=FILE_SIGNATURE)
          ErrorExit("Invalid file format : %s",ghoulfilename);
        
        DOSRead16(TurnsNotIn,RACE_NR,ghoulfile);
        DOSRead16(KillInPhase2,RACE_NR,ghoulfile);
        DOSRead16(&phase1turn,1,ghoulfile);
        DOSRead16(&phase2turn,1,ghoulfile);
     }
    else
      ErrorExit("Cannot read file %s in game dir.",ghoulfilename);

    if (ghoulfile!=NULL) fclose(ghoulfile);
}

void SaveGhoulFile(void)
{
    FILE *ghoulfile;
    Uns32 sygnature;

    sygnature=FILE_SIGNATURE;
    ghoulfile=OpenOutputFile(ghoulfilename,GAME_DIR_ONLY);
    if (ghoulfile!=NULL) {
      DOSWrite32(&sygnature,1,ghoulfile);
      DOSWrite16(TurnsNotIn  ,RACE_NR,ghoulfile);
      DOSWrite16(KillInPhase2,RACE_NR,ghoulfile);
      DOSWrite16(&phase1turn,1,ghoulfile);
      DOSWrite16(&phase2turn,1,ghoulfile);
     }
    else
      ErrorExit("Cannot write file %s in game dir.",ghoulfilename);

    if (ghoulfile!=NULL) fclose(ghoulfile);
}


int
main(int argc, char *argv[])
{
        FILE *ghoulfile;
        FILE *trnfile;
        char tmp[200];
        char killstring[200];
        char tmp1[200];
        char messout[5000];
        int i,t;
        Uns16 RandSeed;
        Uns16 killdone;

        processOptions(argc, argv);
        if (logF==True)
        {
          if ((phase1)&&(phase2))  gLogFile= OpenOutputFile(logname,GAME_DIR_ONLY|TEXT_MODE);
          if ((phase1)&&(!phase2)) gLogFile= OpenOutputFile(logname,GAME_DIR_ONLY|TEXT_MODE);
          if ((!phase1)&&(phase2)) gLogFile= OpenOutputFile(logname,GAME_DIR_ONLY|TEXT_MODE|APPEND_MODE);
        }

        if (printO==True) printOptions(argc, argv);
        ReadConfig();

   version();
   InitPHOSTLib();

/*   If number of ships is 0 (after master)
     and there is no InitializeShips(), gamedata cannot be save.
     Looks like pdklib bug.  */
        InitializeShips();

// now read data
        if (! ReadGlobalData()) ErrorExit("Cannot read global data!");
        if (! ReadHostData())   ErrorExit("Cannot read host data!");

// check if TurnNumber() ==0 if so clear info in paaghul file

        if (TurnNumber()==0)
        {
             memset(TurnsNotIn  , 0, sizeof(TurnsNotIn));
             memset(KillInPhase2, 0, sizeof(KillInPhase2));
             phase1turn=0;
             phase2turn=0;
             SaveGhoulFile();

             // clear kill.log file too

             ghoulfile=OpenOutputFile(killlog,GAME_DIR_ONLY|TEXT_MODE);
             if (ghoulfile) fclose(ghoulfile);

             // send messages to all saying Hello
             strcpy(messout,MessHeadString);
             strcat(messout,"\r");
             sprintf(tmp,"This game uses "MYNAME" v%u.%u%c add-on.\r",pMajor,pMinor,pLetter);
             strcat(messout,tmp);
             strcat(messout,"\r");
             strcat(messout,"Ask your host person for configuration\n");
             strcat(messout,"of this add-on.\n");
             for (i = 1; i <= RACE_NR; i++)
             if (PlayerIsActive(i)) WriteExternalMessage(i,messout);
        }

// setting random seed
        RandSeed=(Uns16) time(0);
        SetRandomSeed(RandSeed);

// display
        if (disp)
        {

// open paaghoul file, in gamedir
        ghoulfile=OpenInputFile(ghoulfilename,GAME_DIR_ONLY|NO_MISSING_ERROR);
        if (ghoulfile==NULL)
        {
          Info("No config file : %s",ghoulfilename);
        }
        else
        {
        fclose(ghoulfile);

        LoadGhoulFile();
        
        Info("Current status : ");
        Info("");
        if (TurnNumber()==0)
        {
          Info("Game newly mastered. No trn info.");
          Info("");
        }
        else
        {
          Info("Turn number         : %d",TurnNumber());

          Info("Phase 1 run on turn : %d",phase1turn);
          Info("Phase 2 run on turn : %d",phase2turn);
          Info("");
        
        for (i=0; i< RACE_NR; i++)
        {
// check is player still active
         if (PlayerIsActive(i+1))
         {
         strcpy(killstring,(KillPlayers?"Will be killed this turn !!":"Should be killed this turn !!"));

         if (TurnsNotIn[i]==0)
             Info("Player %2d. Turn file OK.             %s",i+1,(KillInPhase2[i]==1?killstring:"Status OK."));
          else
             Info("Player %2d. No trn file for %2d turn%s %s",
                  i+1,
                  TurnsNotIn[i],
                  (TurnsNotIn[i]!=1?"s.":". "),
                  (KillInPhase2[i]==1?killstring:"Status OK."));
         }
         else
             Info("Player %2d. Not active in game.",i+1);

        } // for RACE_NR
        } // turn number !=0
         } // paaghoul file exists
        } // disp
        else
        {
// open paaghoul file, in gamedir
        ghoulfile=OpenInputFile(ghoulfilename,GAME_DIR_ONLY|NO_MISSING_ERROR);
        if (ghoulfile==NULL)
            {
// no file, make one
             memset(TurnsNotIn  , 0, sizeof(TurnsNotIn));
             memset(KillInPhase2, 0, sizeof(KillInPhase2));
             phase1turn=0;
             phase2turn=0;
             SaveGhoulFile();
             }
        else
          fclose(ghoulfile);

        LoadGhoulFile();

        if (TurnNumber()==0)
        {
          Info("Game newly mastered. No need to run PAAGHOUL.");
          Info("");
          exit(EXIT_SUCCESS);
        }

        if (phase1)
        {
        Info("Phase 1.");
        Info("");

        if (phase1turn==TurnNumber())
        {
        Info("PAAGHOUL Phase 1 allready run in this turn.");
        Info("");
        }
        else
        {
// now try if playerXX.trn are in GameDir
        for (i=0; i< RACE_NR; i++)
        {
// check is player still active
         if (PlayerIsActive(i+1))
         {
            sprintf(tmp1,"player%d.trn",i+1);
            trnfile=OpenInputFile(tmp1,GAME_DIR_ONLY|NO_MISSING_ERROR);
            if (trnfile==NULL)
            {
            TurnsNotIn[i]++;
            if (TurnsNotIn[i]==1)
            {
               Info("Player %2d. No trn file.",i+1);
               if (WarningMessages==True)
               {
               strcpy(messout,MessHeadString);
               strcat(messout,"\r");
               strcat(messout,"You have not submited trn file\r");
               strcat(messout,"for last turn.\r");
               WriteExternalMessage(i+1,messout);
               }
            } //  ( TurnsNotIn[i]==1)

            if ((TurnsNotIn[i]<TurnsNoSubmitted)&&(TurnsNotIn[i]!=1))
            {
               Info("Player %2d. No trn file for %d turns.",i+1,TurnsNotIn[i]);
               if (WarningMessages==True)
               {
               strcpy(messout,MessHeadString);
               strcat(messout,"\r");
               strcat(messout,"You have not submited\r");
               strcat(messout,"trn file for ");
               sprintf(tmp,"%d",TurnsNotIn[i]);
               strcat(messout,tmp);
               strcat(messout," turns.\r\r");
               strcat(messout,"Waiting ");
               sprintf(tmp,"%d",TurnsNoSubmitted-TurnsNotIn[i]);
               strcat(messout,tmp);
               if ((TurnsNoSubmitted-TurnsNotIn[i])!=1)
                 strcat(messout," more turns to kill.\r");
               else
                 strcat(messout," more turn to kill.\r");
               WriteExternalMessage(i+1,messout);
               }
            }

            if (TurnsNotIn[i]>=TurnsNoSubmitted)
               Info("Player %2d. %s",i+1,(KillPlayers?"Will be killed this turn !!":"Should be killed this turn !!"));

            if (TurnsNotIn[i]>=TurnsNoSubmitted) KillInPhase2[i]=1;
            else                                 KillInPhase2[i]=0;

           } // TRNFILE != NULL
           else
             {
               TurnsNotIn[i]=0;
               KillInPhase2[i]=0;
               Info("Player %2d. Turn file OK.",i+1);

               if (WarningMessages==True)
               {
               strcpy(messout,MessHeadString);
               strcat(messout,"\r");
               strcat(messout,"Thank you for trn file.\r");
               WriteExternalMessage(i+1,messout);
               }
            
            fclose(trnfile);
       }
          }  // player is active
          else
          {
            KillInPhase2[i]=0;
            Info("Player %2d. Not active in game.",i+1);
          }
       }  // for race

// write paaghoul file, in gamedir

         phase1turn=TurnNumber();
         SaveGhoulFile();
         
      } //phase1turn!=turnnumber
         Info("");
     } // phase 1


        if (phase2)
        {
        Info("Phase 2.");
        Info("");

        ghoulfile=OpenInputFile(ghoulfilename,GAME_DIR_ONLY|NO_MISSING_ERROR);
        if (ghoulfile==NULL)
            {
// no file, make one
             memset(TurnsNotIn  , 0, sizeof(TurnsNotIn));
             memset(KillInPhase2, 0, sizeof(KillInPhase2));
             phase1turn=0;
             phase2turn=0;
             SaveGhoulFile();
             }
        else
          fclose(ghoulfile);

        LoadGhoulFile();

        if (phase2turn==TurnNumber())
        {
        Info("PAAGHOUL Phase 2 allready run in this turn.");
        Info("");
        }
        else
        {
        killdone=0;
        for (i=0; i< RACE_NR; i++)
        {
// check is player still active
         if (PlayerIsActive(i+1))
         {
            if (KillInPhase2[i]==1)
            {
             killdone=1;
// truly kill players
             if (KillPlayers==True)
               {
               TurnsNotIn[i]=TurnsNoSubmitted;
               if (killrace(i+1)==0)
               {
               if (KillMessages==True)
               {
// send kill message only when kill is done
               strcpy(messout,MessHeadString);
               strcat(messout,"\r");
               strcat(messout,"Player ");
               sprintf(tmp,"%d",i+1);
               strcat(messout,tmp);
               strcat(messout," has been killed.\r\r");
// info what was killed
               if (KillShips)
                   strcat(messout,"Killing ships.\r");
               if (KillPlanets)
               {
               strcat(messout,"Killing planets.\r");
               if (KillCash)
                   strcat(messout,"Killing cash.\r");
               if (KillStructures)
                   strcat(messout,"Killing structures.\r");
               } // killplanets==true
               if (KillMineFields)
                   strcat(messout,"Killing minefields.\r");
               strcat(messout,"Killing alliances.\r\r");

               sprintf(tmp,"%d",TurnsNoSubmitted);
               strcat(messout,"For ");
               strcat(messout,tmp);
               strcat(messout," turns there was no trn files.\r");
// messages to all active players
               for (t=1;t<12;t++)
               if (PlayerIsActive(t)) WriteExternalMessage(t,messout);
               }// if killmess == True
               }// if killrace == 0
               }//if (KillPlayers==True)
               else
               // killplayers==false
               {
                Info("Player %2d should be killed ! No trn for %d turns.",i+1,TurnsNotIn[i]);
                if (KillMessages==True)
                {
 // send kill message only when kill is done
                strcpy(messout,MessHeadString);
                strcat(messout,"\r");
                strcat(messout,"Player ");
                sprintf(tmp,"%d",i+1);
                strcat(messout,tmp);
                strcat(messout," should be killed.\r\r");
                sprintf(tmp,"%d",TurnsNoSubmitted);
                strcat(messout,"No trn for ");
                strcat(messout,tmp);
                strcat(messout," turns.\r");
 // messages to all active players
                for (t=1;t<12;t++)
                if (PlayerIsActive(t)) WriteExternalMessage(t,messout);
                } // killmess==true
               }  // killplayer==false

              } // designated to kill
              }  // active race
             KillInPhase2[i]=0;
            }  // race

// still phase 2
// write trn info message to all active players if TrnInfoMessages
         if (TrnInfoMessages)
         {
            strcpy(messout,MessHeadString);
            strcat(messout,"\r");
            for (t=1;t<12;t++)
            {
            strcat(messout,"Player ");
            sprintf(tmp,"%2d. ",t);
            strcat(messout,tmp);
            if (PlayerIsActive(t))
            {
              if (TurnsNotIn[t-1]==0)
                strcat(messout,"Trn submitted.\r");
              else
              if (TurnsNotIn[t-1]>=TurnsNoSubmitted)
                strcat(messout,(KillPlayers?"Killed this turn !!\r":"Should be killed !!\r"));
              else
                {
                sprintf(tmp,"No %d of %d trns to kill.\r",TurnsNotIn[t-1],TurnsNoSubmitted);
                strcat(messout,tmp);
                }
            }
            else // player is not active
                strcat(messout,"Is not active.\r");
            } //for
            // send trn info
            for (t=1;t<12;t++)
            if (PlayerIsActive(t)) WriteExternalMessage(t,messout);
         }// trn info mess

// write paaghoul file, in gamedir
         phase2turn=TurnNumber();

         SaveGhoulFile();

         if (killdone==0) Info("Nothing to do.");
      } //phase2turn==turnnumber
         Info("");
    } // phase 2


    if (!WriteHostData()) ErrorExit("Cannot save host data!");
    }  // !disp ==  normal operation

    FreePHOSTLib();
    if (logF==True)
      fclose(gLogFile);

    return EXIT_SUCCESS;
}

/*
**  Portable, public domain replacements for strupr() & strlwr() by Bob Stout
*/

char *strupr(char *string)
{
  char *s;

  if (string)
  {
        for (s = string; *s; ++s)
              *s = toupper(*s);
  }
  return string;
}

